"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var javascriptutilities_1 = require("javascriptutilities");
/**
 * Convenience method to create an action with a default name.
 * @template T Generics parameter.
 * @param {ActionType<T>} action An ActionType instance.
 * @returns {action.Type<T>} An Action type instance.
 */
function createAction(action) {
    if (javascriptutilities_1.Types.isInstance(action, 'name', 'value')) {
        return action;
    }
    else {
        return { name: 'DummyAction', value: action };
    }
}
/**
 * Create reducer streams that can be used to calculate states. Beware that the
 * resulting Observable emits a function that takes a state and returns another
 * state. The value stream from the original Observable is responsible for
 * providing values to the reducer.
 * @template State Generics parameter.
 * @template T Generics parameter.
 * @param {Observable<ActionType<T>>} obs An Observable instance.
 * @param {Reducer<State, T>} reducer A Reducer instance.
 * @returns {Observable<RxReducer<State, T>>} An Observable instance.
 */
function createReducer(obs, reducer) {
    return obs.pipe(operators_1.map(function (v) { return function (state) {
        var action = createAction(v);
        return { state: reducer(state, action), lastAction: createAction(action) };
    }; }));
}
exports.createReducer = createReducer;
/**
 * Create a Redux store that accepts an Array of reducer streams, then scan them
 * to derive the latest state.
 *
 * The setup should be as follows:
 *  - Define BehaviorSubject instances that accept state values.
 *  - Create reducer streams using createReducer.
 *  - Pass the resulting reducer streams to this method.
 *  - When a new state value should be updated, call next(value).
 *  - The new state will be calculated and pushed onto the store stream.
 * @template State Generics paramter.
 * @template T Generics parameter.
 * @param {State} initialState Initial state.
 * @param {Scheduler} scheduler A Scheduler instance.
 * @param {...Observable<RxReducer<State, T>>[]} reducers An Array of Observable.
 * @returns {Observable<stateinfo.Type<State, T>>} An Observable instance.
 */
function create(initialState, scheduler) {
    var reducers = [];
    for (var _i = 2; _i < arguments.length; _i++) {
        reducers[_i - 2] = arguments[_i];
    }
    return rxjs_1.merge.apply(void 0, reducers).pipe(operators_1.scan(function (v1, v2) {
        return v2(v1.state);
    }, { state: initialState, lastAction: undefined }), operators_1.startWith({ state: initialState, lastAction: undefined }), operators_1.observeOn(scheduler));
}
exports.create = create;
/**
 * This store is optional. It only provides some convenience when dealing with
 * state streams.
 * @implements {Type} Type implementation.
 * @template State Generics paramter.
 */
var Self = /** @class */ (function () {
    function Self(initialState, scheduler) {
        var reducers = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            reducers[_i - 2] = arguments[_i];
        }
        this.store = create.apply(void 0, [initialState, scheduler].concat(reducers));
    }
    Object.defineProperty(Self.prototype, "stateInfoStream", {
        get: function () {
            return this.store;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Self.prototype, "stateStream", {
        get: function () {
            return this.store.pipe(operators_1.map(function (v) { return v.state; }));
        },
        enumerable: true,
        configurable: true
    });
    return Self;
}());
exports.Self = Self;
//# sourceMappingURL=rx.js.map